# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = [
    "SessionGetParticipantDataFromPeerIDResponse",
    "Data",
    "DataParticipant",
    "DataParticipantPeerReport",
    "DataParticipantPeerReportMetadata",
    "DataParticipantPeerReportMetadataBrowserMetadata",
    "DataParticipantPeerReportMetadataCandidatePairs",
    "DataParticipantPeerReportMetadataCandidatePairsProducingTransport",
    "DataParticipantPeerReportMetadataDeviceInfo",
    "DataParticipantPeerReportMetadataEvent",
    "DataParticipantPeerReportMetadataIPInformation",
    "DataParticipantPeerReportMetadataIPInformationASN",
    "DataParticipantPeerReportMetadataPcMetadata",
    "DataParticipantPeerReportQuality",
    "DataParticipantPeerReportQualityAudioProducer",
    "DataParticipantPeerReportQualityAudioProducerCumulative",
    "DataParticipantPeerReportQualityAudioProducerCumulativePacketLoss",
    "DataParticipantPeerReportQualityAudioProducerCumulativeQualityMos",
    "DataParticipantPeerReportQualityAudioProducerCumulativeRTT",
    "DataParticipantPeerStats",
    "DataParticipantPeerStatsDeviceInfo",
    "DataParticipantPeerStatsEvent",
    "DataParticipantPeerStatsEventMetadata",
    "DataParticipantPeerStatsEventMetadataConnectionInfo",
    "DataParticipantPeerStatsEventMetadataConnectionInfoConnectivity",
    "DataParticipantPeerStatsEventMetadataConnectionInfoIPDetails",
    "DataParticipantPeerStatsEventMetadataConnectionInfoIPDetailsASN",
    "DataParticipantPeerStatsEventMetadataConnectionInfoLocation",
    "DataParticipantPeerStatsEventMetadataConnectionInfoLocationCoords",
    "DataParticipantPeerStatsIPInformation",
    "DataParticipantPeerStatsIPInformationASN",
    "DataParticipantPeerStatsPrecallNetworkInformation",
    "DataParticipantQualityStats",
]


class DataParticipantPeerReportMetadataBrowserMetadata(BaseModel):
    browser: Optional[str] = None

    browser_version: Optional[str] = None

    engine: Optional[str] = None

    user_agent: Optional[str] = None

    webgl_support: Optional[str] = None


class DataParticipantPeerReportMetadataCandidatePairsProducingTransport(BaseModel):
    available_outgoing_bitrate: Optional[int] = None

    bytes_discarded_on_send: Optional[int] = None

    bytes_received: Optional[int] = None

    bytes_sent: Optional[int] = None

    current_round_trip_time: Optional[float] = None

    last_packet_received_timestamp: Optional[int] = None

    last_packet_sent_timestamp: Optional[int] = None

    local_candidate_address: Optional[str] = None

    local_candidate_id: Optional[str] = None

    local_candidate_network_type: Optional[str] = None

    local_candidate_port: Optional[int] = None

    local_candidate_protocol: Optional[str] = None

    local_candidate_related_address: Optional[str] = None

    local_candidate_related_port: Optional[int] = None

    local_candidate_type: Optional[str] = None

    nominated: Optional[bool] = None

    packets_discarded_on_send: Optional[int] = None

    packets_received: Optional[int] = None

    packets_sent: Optional[int] = None

    remote_candidate_address: Optional[str] = None

    remote_candidate_id: Optional[str] = None

    remote_candidate_port: Optional[int] = None

    remote_candidate_protocol: Optional[str] = None

    remote_candidate_type: Optional[str] = None

    total_round_trip_time: Optional[float] = None


class DataParticipantPeerReportMetadataCandidatePairs(BaseModel):
    consuming_transport: Optional[List[object]] = None

    producing_transport: Optional[List[DataParticipantPeerReportMetadataCandidatePairsProducingTransport]] = None


class DataParticipantPeerReportMetadataDeviceInfo(BaseModel):
    cpus: Optional[int] = None

    is_mobile: Optional[bool] = None

    os: Optional[str] = None

    os_version: Optional[str] = None


class DataParticipantPeerReportMetadataEvent(BaseModel):
    name: Optional[str] = None

    timestamp: Optional[str] = None


class DataParticipantPeerReportMetadataIPInformationASN(BaseModel):
    asn: Optional[str] = None


class DataParticipantPeerReportMetadataIPInformation(BaseModel):
    asn: Optional[DataParticipantPeerReportMetadataIPInformationASN] = None

    city: Optional[str] = None

    country: Optional[str] = None

    ipv4: Optional[str] = None

    region: Optional[str] = None

    timezone: Optional[str] = None


class DataParticipantPeerReportMetadataPcMetadata(BaseModel):
    effective_network_type: Optional[str] = None

    reflexive_connectivity: Optional[bool] = None

    relay_connectivity: Optional[bool] = None

    timestamp: Optional[str] = None

    turn_connectivity: Optional[bool] = None


class DataParticipantPeerReportMetadata(BaseModel):
    audio_devices_updates: Optional[List[object]] = None

    browser_metadata: Optional[DataParticipantPeerReportMetadataBrowserMetadata] = None

    candidate_pairs: Optional[DataParticipantPeerReportMetadataCandidatePairs] = None

    device_info: Optional[DataParticipantPeerReportMetadataDeviceInfo] = None

    events: Optional[List[DataParticipantPeerReportMetadataEvent]] = None

    ip_information: Optional[DataParticipantPeerReportMetadataIPInformation] = None

    pc_metadata: Optional[List[DataParticipantPeerReportMetadataPcMetadata]] = None

    room_view_type: Optional[str] = None

    sdk_name: Optional[str] = None

    sdk_version: Optional[str] = None

    selected_device_updates: Optional[List[object]] = None

    speaker_devices_updates: Optional[List[object]] = None

    video_devices_updates: Optional[List[object]] = None


class DataParticipantPeerReportQualityAudioProducer(BaseModel):
    bytes_sent: Optional[int] = None

    jitter: Optional[int] = None

    mid: Optional[str] = None

    mos_quality: Optional[int] = None

    packets_lost: Optional[int] = None

    packets_sent: Optional[int] = None

    producer_id: Optional[str] = None

    rtt: Optional[float] = None

    ssrc: Optional[int] = None

    timestamp: Optional[str] = None


class DataParticipantPeerReportQualityAudioProducerCumulativePacketLoss(BaseModel):
    api_10_or_greater_event_fraction: Optional[int] = FieldInfo(alias="10_or_greater_event_fraction", default=None)

    api_25_or_greater_event_fraction: Optional[int] = FieldInfo(alias="25_or_greater_event_fraction", default=None)

    api_5_or_greater_event_fraction: Optional[int] = FieldInfo(alias="5_or_greater_event_fraction", default=None)

    api_50_or_greater_event_fraction: Optional[int] = FieldInfo(alias="50_or_greater_event_fraction", default=None)

    avg: Optional[int] = None


class DataParticipantPeerReportQualityAudioProducerCumulativeQualityMos(BaseModel):
    avg: Optional[int] = None

    p50: Optional[int] = None

    p75: Optional[int] = None

    p90: Optional[int] = None


class DataParticipantPeerReportQualityAudioProducerCumulativeRTT(BaseModel):
    api_100ms_or_greater_event_fraction: Optional[float] = FieldInfo(
        alias="100ms_or_greater_event_fraction", default=None
    )

    api_250ms_or_greater_event_fraction: Optional[float] = FieldInfo(
        alias="250ms_or_greater_event_fraction", default=None
    )

    api_500ms_or_greater_event_fraction: Optional[float] = FieldInfo(
        alias="500ms_or_greater_event_fraction", default=None
    )

    avg: Optional[float] = None


class DataParticipantPeerReportQualityAudioProducerCumulative(BaseModel):
    packet_loss: Optional[DataParticipantPeerReportQualityAudioProducerCumulativePacketLoss] = None

    quality_mos: Optional[DataParticipantPeerReportQualityAudioProducerCumulativeQualityMos] = None

    rtt: Optional[DataParticipantPeerReportQualityAudioProducerCumulativeRTT] = None


class DataParticipantPeerReportQuality(BaseModel):
    audio_consumer: Optional[List[object]] = None

    audio_consumer_cumulative: Optional[object] = None

    audio_producer: Optional[List[DataParticipantPeerReportQualityAudioProducer]] = None

    audio_producer_cumulative: Optional[DataParticipantPeerReportQualityAudioProducerCumulative] = None

    screenshare_audio_consumer: Optional[List[object]] = None

    screenshare_audio_consumer_cumulative: Optional[object] = None

    screenshare_audio_producer: Optional[List[object]] = None

    screenshare_audio_producer_cumulative: Optional[object] = None

    screenshare_video_consumer: Optional[List[object]] = None

    screenshare_video_consumer_cumulative: Optional[object] = None

    screenshare_video_producer: Optional[List[object]] = None

    screenshare_video_producer_cumulative: Optional[object] = None

    video_consumer: Optional[List[object]] = None

    video_consumer_cumulative: Optional[object] = None

    video_producer: Optional[List[object]] = None

    video_producer_cumulative: Optional[object] = None


class DataParticipantPeerReport(BaseModel):
    metadata: Optional[DataParticipantPeerReportMetadata] = None

    quality: Optional[DataParticipantPeerReportQuality] = None


class DataParticipantPeerStatsDeviceInfo(BaseModel):
    browser: Optional[str] = None

    browser_version: Optional[str] = None

    cpus: Optional[int] = None

    engine: Optional[str] = None

    is_mobile: Optional[bool] = None

    os: Optional[str] = None

    os_version: Optional[str] = None

    sdk_name: Optional[str] = None

    sdk_version: Optional[str] = None

    user_agent: Optional[str] = None

    webgl_support: Optional[str] = None


class DataParticipantPeerStatsEventMetadataConnectionInfoConnectivity(BaseModel):
    host: Optional[bool] = None

    reflexive: Optional[bool] = None

    relay: Optional[bool] = None


class DataParticipantPeerStatsEventMetadataConnectionInfoIPDetailsASN(BaseModel):
    asn: Optional[str] = None


class DataParticipantPeerStatsEventMetadataConnectionInfoIPDetails(BaseModel):
    asn: Optional[DataParticipantPeerStatsEventMetadataConnectionInfoIPDetailsASN] = None

    city: Optional[str] = None

    country: Optional[str] = None

    ip: Optional[str] = None

    loc: Optional[str] = None

    postal: Optional[str] = None

    region: Optional[str] = None

    timezone: Optional[str] = None


class DataParticipantPeerStatsEventMetadataConnectionInfoLocationCoords(BaseModel):
    latitude: Optional[float] = None

    longitude: Optional[float] = None


class DataParticipantPeerStatsEventMetadataConnectionInfoLocation(BaseModel):
    coords: Optional[DataParticipantPeerStatsEventMetadataConnectionInfoLocationCoords] = None


class DataParticipantPeerStatsEventMetadataConnectionInfo(BaseModel):
    backend_r_t_t: Optional[float] = None

    connectivity: Optional[DataParticipantPeerStatsEventMetadataConnectionInfoConnectivity] = None

    effective_network_type: Optional[str] = None

    fractional_loss: Optional[int] = None

    ip_details: Optional[DataParticipantPeerStatsEventMetadataConnectionInfoIPDetails] = None

    jitter: Optional[int] = None

    location: Optional[DataParticipantPeerStatsEventMetadataConnectionInfoLocation] = None

    r_t_t: Optional[float] = None

    throughput: Optional[int] = None

    turn_connectivity: Optional[bool] = None


class DataParticipantPeerStatsEventMetadata(BaseModel):
    connection_info: Optional[DataParticipantPeerStatsEventMetadataConnectionInfo] = None


class DataParticipantPeerStatsEvent(BaseModel):
    metadata: Optional[DataParticipantPeerStatsEventMetadata] = None

    timestamp: Optional[str] = None

    type: Optional[str] = None


class DataParticipantPeerStatsIPInformationASN(BaseModel):
    asn: Optional[str] = None


class DataParticipantPeerStatsIPInformation(BaseModel):
    asn: Optional[DataParticipantPeerStatsIPInformationASN] = None

    city: Optional[str] = None

    country: Optional[str] = None

    ip_location: Optional[str] = None

    ipv4: Optional[str] = None

    org: Optional[str] = None

    region: Optional[str] = None

    timezone: Optional[str] = None


class DataParticipantPeerStatsPrecallNetworkInformation(BaseModel):
    backend_rtt: Optional[float] = None

    effective_networktype: Optional[str] = None

    fractional_loss: Optional[int] = None

    jitter: Optional[int] = None

    reflexive_connectivity: Optional[bool] = None

    relay_connectivity: Optional[bool] = None

    rtt: Optional[float] = None

    throughput: Optional[int] = None

    turn_connectivity: Optional[bool] = None


class DataParticipantPeerStats(BaseModel):
    device_info: Optional[DataParticipantPeerStatsDeviceInfo] = None

    events: Optional[List[DataParticipantPeerStatsEvent]] = None

    ip_information: Optional[DataParticipantPeerStatsIPInformation] = None

    precall_network_information: Optional[DataParticipantPeerStatsPrecallNetworkInformation] = None


class DataParticipantQualityStats(BaseModel):
    audio_bandwidth: Optional[int] = None

    audio_stats: Optional[List[object]] = None

    average_quality: Optional[int] = None

    end: Optional[str] = None

    first_audio_packet_received: Optional[str] = None

    first_video_packet_received: Optional[str] = None

    last_audio_packet_received: Optional[str] = None

    last_video_packet_received: Optional[str] = None

    peer_ids: Optional[List[str]] = None

    start: Optional[str] = None

    total_audio_packets: Optional[int] = None

    total_audio_packets_lost: Optional[int] = None

    total_video_packets: Optional[int] = None

    total_video_packets_lost: Optional[int] = None

    video_bandwidth: Optional[int] = None

    video_stats: Optional[List[object]] = None


class DataParticipant(BaseModel):
    id: Optional[str] = None

    created_at: Optional[str] = None

    custom_participant_id: Optional[str] = None

    display_name: Optional[str] = None

    duration: Optional[float] = None

    joined_at: Optional[str] = None

    left_at: Optional[str] = None

    peer_report: Optional[DataParticipantPeerReport] = None

    peer_stats: Optional[DataParticipantPeerStats] = None

    quality_stats: Optional[DataParticipantQualityStats] = None

    role: Optional[str] = None

    updated_at: Optional[str] = None

    user_id: Optional[str] = None


class Data(BaseModel):
    participant: Optional[DataParticipant] = None


class SessionGetParticipantDataFromPeerIDResponse(BaseModel):
    data: Optional[Data] = None

    success: Optional[bool] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, TypeAlias

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = [
    "DynamicRoutingUpdateResponse",
    "Route",
    "RouteDeployment",
    "RouteElement",
    "RouteElementUnionMember0",
    "RouteElementUnionMember0Outputs",
    "RouteElementUnionMember0OutputsNext",
    "RouteElementUnionMember1",
    "RouteElementUnionMember1Outputs",
    "RouteElementUnionMember1OutputsFalse_",
    "RouteElementUnionMember1OutputsTrue_",
    "RouteElementUnionMember1Properties",
    "RouteElementUnionMember2",
    "RouteElementUnionMember2Outputs",
    "RouteElementUnionMember3",
    "RouteElementUnionMember3Outputs",
    "RouteElementUnionMember3OutputsFallback",
    "RouteElementUnionMember3OutputsSuccess",
    "RouteElementUnionMember3Properties",
    "RouteElementUnionMember4",
    "RouteElementUnionMember4Outputs",
    "RouteElementUnionMember4OutputsFallback",
    "RouteElementUnionMember4OutputsSuccess",
    "RouteElementUnionMember4Properties",
    "RouteElementUnionMember5",
    "RouteElementUnionMember5Outputs",
    "RouteVersion",
]


class RouteDeployment(BaseModel):
    created_at: str

    deployment_id: str

    version_id: str


class RouteElementUnionMember0OutputsNext(BaseModel):
    element_id: str = FieldInfo(alias="elementId")


class RouteElementUnionMember0Outputs(BaseModel):
    next: RouteElementUnionMember0OutputsNext


class RouteElementUnionMember0(BaseModel):
    id: str

    outputs: RouteElementUnionMember0Outputs

    type: Literal["start"]


class RouteElementUnionMember1OutputsFalse_(BaseModel):
    element_id: str = FieldInfo(alias="elementId")


class RouteElementUnionMember1OutputsTrue_(BaseModel):
    element_id: str = FieldInfo(alias="elementId")


class RouteElementUnionMember1Outputs(BaseModel):
    false: RouteElementUnionMember1OutputsFalse_

    true: RouteElementUnionMember1OutputsTrue_


class RouteElementUnionMember1Properties(BaseModel):
    conditions: Optional[object] = None


class RouteElementUnionMember1(BaseModel):
    id: str

    outputs: RouteElementUnionMember1Outputs

    properties: RouteElementUnionMember1Properties

    type: Literal["conditional"]


class RouteElementUnionMember2Outputs(BaseModel):
    element_id: str = FieldInfo(alias="elementId")


class RouteElementUnionMember2(BaseModel):
    id: str

    outputs: Dict[str, RouteElementUnionMember2Outputs]

    type: Literal["percentage"]


class RouteElementUnionMember3OutputsFallback(BaseModel):
    element_id: str = FieldInfo(alias="elementId")


class RouteElementUnionMember3OutputsSuccess(BaseModel):
    element_id: str = FieldInfo(alias="elementId")


class RouteElementUnionMember3Outputs(BaseModel):
    fallback: RouteElementUnionMember3OutputsFallback

    success: RouteElementUnionMember3OutputsSuccess


class RouteElementUnionMember3Properties(BaseModel):
    key: str

    limit: float

    limit_type: Literal["count", "cost"] = FieldInfo(alias="limitType")

    window: float


class RouteElementUnionMember3(BaseModel):
    id: str

    outputs: RouteElementUnionMember3Outputs

    properties: RouteElementUnionMember3Properties

    type: Literal["rate"]


class RouteElementUnionMember4OutputsFallback(BaseModel):
    element_id: str = FieldInfo(alias="elementId")


class RouteElementUnionMember4OutputsSuccess(BaseModel):
    element_id: str = FieldInfo(alias="elementId")


class RouteElementUnionMember4Outputs(BaseModel):
    fallback: RouteElementUnionMember4OutputsFallback

    success: RouteElementUnionMember4OutputsSuccess


class RouteElementUnionMember4Properties(BaseModel):
    model: str

    provider: str

    retries: float

    timeout: float


class RouteElementUnionMember4(BaseModel):
    id: str

    outputs: RouteElementUnionMember4Outputs

    properties: RouteElementUnionMember4Properties

    type: Literal["model"]


class RouteElementUnionMember5Outputs(BaseModel):
    element_id: str = FieldInfo(alias="elementId")


class RouteElementUnionMember5(BaseModel):
    id: str

    outputs: Dict[str, RouteElementUnionMember5Outputs]

    type: Literal["end"]


RouteElement: TypeAlias = Union[
    RouteElementUnionMember0,
    RouteElementUnionMember1,
    RouteElementUnionMember2,
    RouteElementUnionMember3,
    RouteElementUnionMember4,
    RouteElementUnionMember5,
]


class RouteVersion(BaseModel):
    active: Literal["true", "false"]

    created_at: str

    data: str

    version_id: str

    is_valid: Optional[bool] = None


class Route(BaseModel):
    id: str

    account_tag: str

    created_at: datetime

    deployment: RouteDeployment

    elements: List[RouteElement]

    gateway_id: str

    modified_at: datetime

    name: str

    version: RouteVersion


class DynamicRoutingUpdateResponse(BaseModel):
    route: Route

    success: bool



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ..._models import BaseModel

__all__ = ["RegionalHostnameEditResponse"]


class RegionalHostnameEditResponse(BaseModel):
    created_on: datetime
    """When the regional hostname was created"""

    hostname: str
    """DNS hostname to be regionalized, must be a subdomain of the zone.

    Wildcards are supported for one level, e.g `*.example.com`
    """

    region_key: str
    """Identifying key for the region"""

    routing: Optional[str] = None
    """Configure which routing method to use for the regional hostname"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, TypeAlias

from ..._models import BaseModel
from .psk_metadata import PSKMetadata
from .health_check_rate import HealthCheckRate
from .health_check_type import HealthCheckType

__all__ = [
    "IPSECTunnelBulkUpdateResponse",
    "ModifiedIPSECTunnel",
    "ModifiedIPSECTunnelBGP",
    "ModifiedIPSECTunnelBGPStatus",
    "ModifiedIPSECTunnelCustomRemoteIdentities",
    "ModifiedIPSECTunnelHealthCheck",
    "ModifiedIPSECTunnelHealthCheckTarget",
    "ModifiedIPSECTunnelHealthCheckTargetMagicHealthCheckTarget",
]


class ModifiedIPSECTunnelBGP(BaseModel):
    customer_asn: int
    """ASN used on the customer end of the BGP session"""

    extra_prefixes: Optional[List[str]] = None
    """
    Prefixes in this list will be advertised to the customer device, in addition to
    the routes in the Magic routing table.
    """

    md5_key: Optional[str] = None
    """MD5 key to use for session authentication.

    Note that _this is not a security measure_. MD5 is not a valid security
    mechanism, and the key is not treated as a secret value. This is _only_
    supported for preventing misconfiguration, not for defending against malicious
    attacks.

    The MD5 key, if set, must be of non-zero length and consist only of the
    following types of character:

    - ASCII alphanumerics: `[a-zA-Z0-9]`
    - Special characters in the set `'!@#$%^&*()+[]{}<>/.,;:_-~`= \\||`

    In other words, MD5 keys may contain any printable ASCII character aside from
    newline (0x0A), quotation mark (`"`), vertical tab (0x0B), carriage return
    (0x0D), tab (0x09), form feed (0x0C), and the question mark (`?`). Requests
    specifying an MD5 key with one or more of these disallowed characters will be
    rejected.
    """


class ModifiedIPSECTunnelBGPStatus(BaseModel):
    state: Literal["BGP_DOWN", "BGP_UP", "BGP_ESTABLISHING"]

    tcp_established: bool

    updated_at: datetime

    bgp_state: Optional[str] = None

    cf_speaker_ip: Optional[str] = None

    cf_speaker_port: Optional[int] = None

    customer_speaker_ip: Optional[str] = None

    customer_speaker_port: Optional[int] = None


class ModifiedIPSECTunnelCustomRemoteIdentities(BaseModel):
    fqdn_id: Optional[str] = None
    """A custom IKE ID of type FQDN that may be used to identity the IPsec tunnel.

    The generated IKE IDs can still be used even if this custom value is specified.

    Must be of the form `<custom label>.<account ID>.custom.ipsec.cloudflare.com`.

    This custom ID does not need to be unique. Two IPsec tunnels may have the same
    custom fqdn_id. However, if another IPsec tunnel has the same value then the two
    tunnels cannot have the same cloudflare_endpoint.
    """


class ModifiedIPSECTunnelHealthCheckTargetMagicHealthCheckTarget(BaseModel):
    """The destination address in a request type health check.

    After the healthcheck is decapsulated at the customer end of the tunnel, the ICMP echo will be forwarded to this address. This field defaults to `customer_gre_endpoint address`. This field is ignored for bidirectional healthchecks as the interface_address (not assigned to the Cloudflare side of the tunnel) is used as the target.
    """

    effective: Optional[str] = None
    """The effective health check target.

    If 'saved' is empty, then this field will be populated with the calculated
    default value on GET requests. Ignored in POST, PUT, and PATCH requests.
    """

    saved: Optional[str] = None
    """The saved health check target.

    Setting the value to the empty string indicates that the calculated default
    value will be used.
    """


ModifiedIPSECTunnelHealthCheckTarget: TypeAlias = Union[ModifiedIPSECTunnelHealthCheckTargetMagicHealthCheckTarget, str]


class ModifiedIPSECTunnelHealthCheck(BaseModel):
    direction: Optional[Literal["unidirectional", "bidirectional"]] = None
    """The direction of the flow of the healthcheck.

    Either unidirectional, where the probe comes to you via the tunnel and the
    result comes back to Cloudflare via the open Internet, or bidirectional where
    both the probe and result come and go via the tunnel.
    """

    enabled: Optional[bool] = None
    """Determines whether to run healthchecks for a tunnel."""

    rate: Optional[HealthCheckRate] = None
    """How frequent the health check is run. The default value is `mid`."""

    target: Optional[ModifiedIPSECTunnelHealthCheckTarget] = None
    """The destination address in a request type health check.

    After the healthcheck is decapsulated at the customer end of the tunnel, the
    ICMP echo will be forwarded to this address. This field defaults to
    `customer_gre_endpoint address`. This field is ignored for bidirectional
    healthchecks as the interface_address (not assigned to the Cloudflare side of
    the tunnel) is used as the target. Must be in object form if the
    x-magic-new-hc-target header is set to true and string form if
    x-magic-new-hc-target is absent or set to false.
    """

    type: Optional[HealthCheckType] = None
    """The type of healthcheck to run, reply or request. The default value is `reply`."""


class ModifiedIPSECTunnel(BaseModel):
    id: str
    """Identifier"""

    cloudflare_endpoint: str
    """The IP address assigned to the Cloudflare side of the IPsec tunnel."""

    interface_address: str
    """
    A 31-bit prefix (/31 in CIDR notation) supporting two hosts, one for each side
    of the tunnel. Select the subnet from the following private IP space:
    10.0.0.0–10.255.255.255, 172.16.0.0–172.31.255.255, 192.168.0.0–192.168.255.255.
    """

    name: str
    """The name of the IPsec tunnel. The name cannot share a name with other tunnels."""

    allow_null_cipher: Optional[bool] = None
    """
    When `true`, the tunnel can use a null-cipher (`ENCR_NULL`) in the ESP tunnel
    (Phase 2).
    """

    automatic_return_routing: Optional[bool] = None
    """
    True if automatic stateful return routing should be enabled for a tunnel, false
    otherwise. Requires the `coupler_integration` account flag to be enabled;
    requests setting this to `true` without that flag will be rejected.
    """

    bgp: Optional[ModifiedIPSECTunnelBGP] = None

    bgp_status: Optional[ModifiedIPSECTunnelBGPStatus] = None

    created_on: Optional[datetime] = None
    """The date and time the tunnel was created."""

    custom_remote_identities: Optional[ModifiedIPSECTunnelCustomRemoteIdentities] = None

    customer_endpoint: Optional[str] = None
    """The IP address assigned to the customer side of the IPsec tunnel.

    Not required, but must be set for proactive traceroutes to work.
    """

    description: Optional[str] = None
    """An optional description forthe IPsec tunnel."""

    health_check: Optional[ModifiedIPSECTunnelHealthCheck] = None

    interface_address6: Optional[str] = None
    """
    A 127 bit IPV6 prefix from within the virtual_subnet6 prefix space with the
    address being the first IP of the subnet and not same as the address of
    virtual_subnet6. Eg if virtual_subnet6 is 2606:54c1:7:0:a9fe:12d2::/127 ,
    interface_address6 could be 2606:54c1:7:0:a9fe:12d2:1:200/127
    """

    modified_on: Optional[datetime] = None
    """The date and time the tunnel was last modified."""

    psk_metadata: Optional[PSKMetadata] = None
    """The PSK metadata that includes when the PSK was generated."""

    replay_protection: Optional[bool] = None
    """
    If `true`, then IPsec replay protection will be supported in the
    Cloudflare-to-customer direction.
    """


class IPSECTunnelBulkUpdateResponse(BaseModel):
    modified: Optional[bool] = None

    modified_ipsec_tunnels: Optional[List[ModifiedIPSECTunnel]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["DNSResolverSettingsV6Param"]


class DNSResolverSettingsV6Param(TypedDict, total=False):
    ip: Required[str]
    """Specify the IPv6 address of the upstream resolver."""

    port: int
    """Specify a port number to use for the upstream resolver.

    Defaults to 53 if unspecified.
    """

    route_through_private_network: bool
    """Indicate whether to connect to this resolver over a private network.

    Must set when vnet_id set.
    """

    vnet_id: str
    """Specify an optional virtual network for this resolver.

    Uses default virtual network id if omitted.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel

__all__ = ["ConfigurationEditResponse"]


class ConfigurationEditResponse(BaseModel):
    id: Optional[str] = None
    """UUID."""

    description: Optional[str] = None

    title: Optional[str] = None

    token_sources: Optional[List[str]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .warp import (
    WARPResource,
    AsyncWARPResource,
    WARPResourceWithRawResponse,
    AsyncWARPResourceWithRawResponse,
    WARPResourceWithStreamingResponse,
    AsyncWARPResourceWithStreamingResponse,
)
from .subnets import (
    SubnetsResource,
    AsyncSubnetsResource,
    SubnetsResourceWithRawResponse,
    AsyncSubnetsResourceWithRawResponse,
    SubnetsResourceWithStreamingResponse,
    AsyncSubnetsResourceWithStreamingResponse,
)
from .cloudflare_source import (
    CloudflareSourceResource,
    AsyncCloudflareSourceResource,
    CloudflareSourceResourceWithRawResponse,
    AsyncCloudflareSourceResourceWithRawResponse,
    CloudflareSourceResourceWithStreamingResponse,
    AsyncCloudflareSourceResourceWithStreamingResponse,
)

__all__ = [
    "WARPResource",
    "AsyncWARPResource",
    "WARPResourceWithRawResponse",
    "AsyncWARPResourceWithRawResponse",
    "WARPResourceWithStreamingResponse",
    "AsyncWARPResourceWithStreamingResponse",
    "CloudflareSourceResource",
    "AsyncCloudflareSourceResource",
    "CloudflareSourceResourceWithRawResponse",
    "AsyncCloudflareSourceResourceWithRawResponse",
    "CloudflareSourceResourceWithStreamingResponse",
    "AsyncCloudflareSourceResourceWithStreamingResponse",
    "SubnetsResource",
    "AsyncSubnetsResource",
    "SubnetsResourceWithRawResponse",
    "AsyncSubnetsResourceWithRawResponse",
    "SubnetsResourceWithStreamingResponse",
    "AsyncSubnetsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["Settings"]


class Settings(BaseModel):
    id: str
    """Email Routing settings identifier."""

    enabled: Literal[True, False]
    """State of the zone settings for Email Routing."""

    name: str
    """Domain of your zone."""

    created: Optional[datetime] = None
    """The date and time the settings have been created."""

    modified: Optional[datetime] = None
    """The date and time the settings have been modified."""

    skip_wizard: Optional[Literal[True, False]] = None
    """Flag to check if the user skipped the configuration wizard."""

    status: Optional[Literal["ready", "unconfigured", "misconfigured", "misconfigured/locked", "unlocked"]] = None
    """Show the state of your account, and the type or configuration error."""

    tag: Optional[str] = None
    """Email Routing settings tag.

    (Deprecated, replaced by Email Routing settings identifier)
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from ...._types import SequenceNotStr

__all__ = ["LabelBulkUpdateParams", "Managed", "Selector", "SelectorInclude", "User"]


class LabelBulkUpdateParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    managed: Required[Managed]
    """Managed labels to replace for all affected operations"""

    selector: Required[Selector]
    """Operation IDs selector"""

    user: Required[User]
    """User labels to replace for all affected operations"""


class Managed(TypedDict, total=False):
    """Managed labels to replace for all affected operations"""

    labels: Required[SequenceNotStr[str]]
    """List of managed label names.

    Providing an empty array will result in all managed labels being removed from
    all affected operations
    """


class SelectorInclude(TypedDict, total=False):
    operation_ids: Required[SequenceNotStr[str]]


class Selector(TypedDict, total=False):
    """Operation IDs selector"""

    include: Required[SelectorInclude]


class User(TypedDict, total=False):
    """User labels to replace for all affected operations"""

    labels: Required[SequenceNotStr[str]]
    """List of user label names.

    Providing an empty array will result in all user labels being removed from all
    affected operations
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["JobUpdateParams"]


class JobUpdateParams(TypedDict, total=False):
    account_id: Required[str]

    name: Required[str]

    id: Required[str]
    """AI Search instance ID. Lowercase alphanumeric, hyphens, and underscores."""

    action: Required[Literal["cancel"]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

from ..._types import SequenceNotStr
from .identity_provider_type import IdentityProviderType
from .identity_provider_scim_config_param import IdentityProviderSCIMConfigParam

__all__ = ["AzureADParam", "Config"]


class Config(TypedDict, total=False):
    """The configuration parameters for the identity provider.

    To view the required parameters for a specific provider, refer to our [developer documentation](https://developers.cloudflare.com/cloudflare-one/identity/idp-integration/).
    """

    claims: SequenceNotStr[str]
    """Custom claims"""

    client_id: str
    """Your OAuth Client ID"""

    client_secret: str
    """Your OAuth Client Secret"""

    conditional_access_enabled: bool
    """Should Cloudflare try to load authentication contexts from your account"""

    directory_id: str
    """Your Azure directory uuid"""

    email_claim_name: str
    """The claim name for email in the id_token response."""

    prompt: Literal["login", "select_account", "none"]
    """Indicates the type of user interaction that is required.

    prompt=login forces the user to enter their credentials on that request,
    negating single-sign on. prompt=none is the opposite. It ensures that the user
    isn't presented with any interactive prompt. If the request can't be completed
    silently by using single-sign on, the Microsoft identity platform returns an
    interaction_required error. prompt=select_account interrupts single sign-on
    providing account selection experience listing all the accounts either in
    session or any remembered account or an option to choose to use a different
    account altogether.
    """

    support_groups: bool
    """Should Cloudflare try to load groups from your account"""


class AzureADParam(TypedDict, total=False):
    config: Required[Config]
    """The configuration parameters for the identity provider.

    To view the required parameters for a specific provider, refer to our
    [developer documentation](https://developers.cloudflare.com/cloudflare-one/identity/idp-integration/).
    """

    name: Required[str]
    """The name of the identity provider, shown to users on the login page."""

    type: Required[IdentityProviderType]
    """The type of identity provider.

    To determine the value for a specific provider, refer to our
    [developer documentation](https://developers.cloudflare.com/cloudflare-one/identity/idp-integration/).
    """

    saml_certificate_set_id: str
    """
    The UID of the SAML encryption certificate set assigned to this Identity
    Provider. Only present for SAML identity providers with encryption configured.
    Create a certificate set via POST to
    `/identity_providers/{id}/saml_certificate`.
    """

    scim_config: IdentityProviderSCIMConfigParam
    """
    The configuration settings for enabling a System for Cross-Domain Identity
    Management (SCIM) with the identity provider.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .top import (
    TopResource,
    AsyncTopResource,
    TopResourceWithRawResponse,
    AsyncTopResourceWithRawResponse,
    TopResourceWithStreamingResponse,
    AsyncTopResourceWithStreamingResponse,
)
from .speed import (
    SpeedResource,
    AsyncSpeedResource,
    SpeedResourceWithRawResponse,
    AsyncSpeedResourceWithRawResponse,
    SpeedResourceWithStreamingResponse,
    AsyncSpeedResourceWithStreamingResponse,
)

__all__ = [
    "TopResource",
    "AsyncTopResource",
    "TopResourceWithRawResponse",
    "AsyncTopResourceWithRawResponse",
    "TopResourceWithStreamingResponse",
    "AsyncTopResourceWithStreamingResponse",
    "SpeedResource",
    "AsyncSpeedResource",
    "SpeedResourceWithRawResponse",
    "AsyncSpeedResourceWithRawResponse",
    "SpeedResourceWithStreamingResponse",
    "AsyncSpeedResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .asns import (
    ASNsResource,
    AsyncASNsResource,
    ASNsResourceWithRawResponse,
    AsyncASNsResourceWithRawResponse,
    ASNsResourceWithStreamingResponse,
    AsyncASNsResourceWithStreamingResponse,
)
from .entities import (
    EntitiesResource,
    AsyncEntitiesResource,
    EntitiesResourceWithRawResponse,
    AsyncEntitiesResourceWithRawResponse,
    EntitiesResourceWithStreamingResponse,
    AsyncEntitiesResourceWithStreamingResponse,
)
from .locations import (
    LocationsResource,
    AsyncLocationsResource,
    LocationsResourceWithRawResponse,
    AsyncLocationsResourceWithRawResponse,
    LocationsResourceWithStreamingResponse,
    AsyncLocationsResourceWithStreamingResponse,
)

__all__ = [
    "ASNsResource",
    "AsyncASNsResource",
    "ASNsResourceWithRawResponse",
    "AsyncASNsResourceWithRawResponse",
    "ASNsResourceWithStreamingResponse",
    "AsyncASNsResourceWithStreamingResponse",
    "LocationsResource",
    "AsyncLocationsResource",
    "LocationsResourceWithRawResponse",
    "AsyncLocationsResourceWithRawResponse",
    "LocationsResourceWithStreamingResponse",
    "AsyncLocationsResourceWithStreamingResponse",
    "EntitiesResource",
    "AsyncEntitiesResource",
    "EntitiesResourceWithRawResponse",
    "AsyncEntitiesResourceWithRawResponse",
    "EntitiesResourceWithStreamingResponse",
    "AsyncEntitiesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .move_bulk_params import MoveBulkParams as MoveBulkParams
from .raw_get_response import RawGetResponse as RawGetResponse
from .move_bulk_response import MoveBulkResponse as MoveBulkResponse
from .move_create_params import MoveCreateParams as MoveCreateParams
from .trace_get_response import TraceGetResponse as TraceGetResponse
from .release_bulk_params import ReleaseBulkParams as ReleaseBulkParams
from .move_create_response import MoveCreateResponse as MoveCreateResponse
from .preview_get_response import PreviewGetResponse as PreviewGetResponse
from .preview_create_params import PreviewCreateParams as PreviewCreateParams
from .release_bulk_response import ReleaseBulkResponse as ReleaseBulkResponse
from .detection_get_response import DetectionGetResponse as DetectionGetResponse
from .preview_create_response import PreviewCreateResponse as PreviewCreateResponse
from .reclassify_create_params import ReclassifyCreateParams as ReclassifyCreateParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast

import httpx

from ..._types import Body, Query, Headers, NotGiven, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ..._base_client import make_request_options
from ...types.waiting_rooms import page_preview_params
from ...types.waiting_rooms.page_preview_response import PagePreviewResponse

__all__ = ["PageResource", "AsyncPageResource"]


class PageResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PageResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return PageResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PageResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return PageResourceWithStreamingResponse(self)

    def preview(
        self,
        *,
        zone_id: str,
        custom_html: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PagePreviewResponse:
        """Creates a waiting room page preview.

        Upload a custom waiting room page for
        preview. You will receive a preview URL in the form
        `http://waitingrooms.dev/preview/<uuid>`. You can use the following query
        parameters to change the state of the preview:

        1. `force_queue`: Boolean indicating if all users will be queued in the waiting
           room and no one will be let into the origin website (also known as queueAll).
        2. `queue_is_full`: Boolean indicating if the waiting room's queue is currently
           full and not accepting new users at the moment.
        3. `queueing_method`: The queueing method currently used by the waiting room.
           - **fifo** indicates a FIFO queue.
           - **random** indicates a Random queue.
           - **passthrough** indicates a Passthrough queue. Keep in mind that the
             waiting room page will only be displayed if `force_queue=true` or
             `event=prequeueing` — for other cases the request will pass through to the
             origin. For our preview, this will be a fake origin website returning
             \"Welcome\".
           - **reject** indicates a Reject queue.
        4. `event`: Used to preview a waiting room event.
           - **none** indicates no event is occurring.
           - **prequeueing** indicates that an event is prequeueing (between
             `prequeue_start_time` and `event_start_time`).
           - **started** indicates that an event has started (between `event_start_time`
             and `event_end_time`).
        5. `shuffle_at_event_start`: Boolean indicating if the event will shuffle users
           in the prequeue when it starts. This can only be set to **true** if an event
           is active (`event` is not **none**).

        For example, you can make a request to
        `http://waitingrooms.dev/preview/<uuid>?force_queue=false&queue_is_full=false&queueing_method=random&event=started&shuffle_at_event_start=true` 6.
        `waitTime`: Non-zero, positive integer indicating the estimated wait time in
        minutes. The default value is 10 minutes.

        For example, you can make a request to
        `http://waitingrooms.dev/preview/<uuid>?waitTime=50` to configure the estimated
        wait time as 50 minutes.

        Args:
          zone_id: Identifier.

          custom_html: Only available for the Waiting Room Advanced subscription. This is a template
              html file that will be rendered at the edge. If no custom_page_html is provided,
              the default waiting room will be used. The template is based on mustache (
              https://mustache.github.io/ ). There are several variables that are evaluated by
              the Cloudflare edge:

              1. {{`waitTimeKnown`}} Acts like a boolean value that indicates the behavior to
                 take when wait time is not available, for instance when queue_all is
                 **true**.
              2. {{`waitTimeFormatted`}} Estimated wait time for the user. For example, five
                 minutes. Alternatively, you can use:
              3. {{`waitTime`}} Number of minutes of estimated wait for a user.
              4. {{`waitTimeHours`}} Number of hours of estimated wait for a user
                 (`Math.floor(waitTime/60)`).
              5. {{`waitTimeHourMinutes`}} Number of minutes above the `waitTimeHours` value
                 (`waitTime%60`).
              6. {{`queueIsFull`}} Changes to **true** when no more people can be added to the
                 queue.

              To view the full list of variables, look at the `cfWaitingRoom` object described
              under the `json_response_enabled` property in other Waiting Room API calls.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._post(
            path_template("/zones/{zone_id}/waiting_rooms/preview", zone_id=zone_id),
            body=maybe_transform({"custom_html": custom_html}, page_preview_params.PagePreviewParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[PagePreviewResponse]._unwrapper,
            ),
            cast_to=cast(Type[PagePreviewResponse], ResultWrapper[PagePreviewResponse]),
        )


class AsyncPageResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPageResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPageResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPageResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncPageResourceWithStreamingResponse(self)

    async def preview(
        self,
        *,
        zone_id: str,
        custom_html: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PagePreviewResponse:
        """Creates a waiting room page preview.

        Upload a custom waiting room page for
        preview. You will receive a preview URL in the form
        `http://waitingrooms.dev/preview/<uuid>`. You can use the following query
        parameters to change the state of the preview:

        1. `force_queue`: Boolean indicating if all users will be queued in the waiting
           room and no one will be let into the origin website (also known as queueAll).
        2. `queue_is_full`: Boolean indicating if the waiting room's queue is currently
           full and not accepting new users at the moment.
        3. `queueing_method`: The queueing method currently used by the waiting room.
           - **fifo** indicates a FIFO queue.
           - **random** indicates a Random queue.
           - **passthrough** indicates a Passthrough queue. Keep in mind that the
             waiting room page will only be displayed if `force_queue=true` or
             `event=prequeueing` — for other cases the request will pass through to the
             origin. For our preview, this will be a fake origin website returning
             \"Welcome\".
           - **reject** indicates a Reject queue.
        4. `event`: Used to preview a waiting room event.
           - **none** indicates no event is occurring.
           - **prequeueing** indicates that an event is prequeueing (between
             `prequeue_start_time` and `event_start_time`).
           - **started** indicates that an event has started (between `event_start_time`
             and `event_end_time`).
        5. `shuffle_at_event_start`: Boolean indicating if the event will shuffle users
           in the prequeue when it starts. This can only be set to **true** if an event
           is active (`event` is not **none**).

        For example, you can make a request to
        `http://waitingrooms.dev/preview/<uuid>?force_queue=false&queue_is_full=false&queueing_method=random&event=started&shuffle_at_event_start=true` 6.
        `waitTime`: Non-zero, positive integer indicating the estimated wait time in
        minutes. The default value is 10 minutes.

        For example, you can make a request to
        `http://waitingrooms.dev/preview/<uuid>?waitTime=50` to configure the estimated
        wait time as 50 minutes.

        Args:
          zone_id: Identifier.

          custom_html: Only available for the Waiting Room Advanced subscription. This is a template
              html file that will be rendered at the edge. If no custom_page_html is provided,
              the default waiting room will be used. The template is based on mustache (
              https://mustache.github.io/ ). There are several variables that are evaluated by
              the Cloudflare edge:

              1. {{`waitTimeKnown`}} Acts like a boolean value that indicates the behavior to
                 take when wait time is not available, for instance when queue_all is
                 **true**.
              2. {{`waitTimeFormatted`}} Estimated wait time for the user. For example, five
                 minutes. Alternatively, you can use:
              3. {{`waitTime`}} Number of minutes of estimated wait for a user.
              4. {{`waitTimeHours`}} Number of hours of estimated wait for a user
                 (`Math.floor(waitTime/60)`).
              5. {{`waitTimeHourMinutes`}} Number of minutes above the `waitTimeHours` value
                 (`waitTime%60`).
              6. {{`queueIsFull`}} Changes to **true** when no more people can be added to the
                 queue.

              To view the full list of variables, look at the `cfWaitingRoom` object described
              under the `json_response_enabled` property in other Waiting Room API calls.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._post(
            path_template("/zones/{zone_id}/waiting_rooms/preview", zone_id=zone_id),
            body=await async_maybe_transform({"custom_html": custom_html}, page_preview_params.PagePreviewParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[PagePreviewResponse]._unwrapper,
            ),
            cast_to=cast(Type[PagePreviewResponse], ResultWrapper[PagePreviewResponse]),
        )


class PageResourceWithRawResponse:
    def __init__(self, page: PageResource) -> None:
        self._page = page

        self.preview = to_raw_response_wrapper(
            page.preview,
        )


class AsyncPageResourceWithRawResponse:
    def __init__(self, page: AsyncPageResource) -> None:
        self._page = page

        self.preview = async_to_raw_response_wrapper(
            page.preview,
        )


class PageResourceWithStreamingResponse:
    def __init__(self, page: PageResource) -> None:
        self._page = page

        self.preview = to_streamed_response_wrapper(
            page.preview,
        )


class AsyncPageResourceWithStreamingResponse:
    def __init__(self, page: AsyncPageResource) -> None:
        self._page = page

        self.preview = async_to_streamed_response_wrapper(
            page.preview,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["ConfigurationGetParams"]


class ConfigurationGetParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    normalize: bool
    """Ensures that the configuration is written or retrieved in normalized fashion"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .label import Label as Label
from .message import Message as Message
from .priority import Priority as Priority
from .asset_get_response import AssetGetResponse as AssetGetResponse
from .message_get_params import MessageGetParams as MessageGetParams
from .asset_create_params import AssetCreateParams as AssetCreateParams
from .asset_update_params import AssetUpdateParams as AssetUpdateParams
from .asset_create_response import AssetCreateResponse as AssetCreateResponse
from .asset_delete_response import AssetDeleteResponse as AssetDeleteResponse
from .asset_update_response import AssetUpdateResponse as AssetUpdateResponse
from .message_create_params import MessageCreateParams as MessageCreateParams
from .message_update_params import MessageUpdateParams as MessageUpdateParams
from .priority_create_params import PriorityCreateParams as PriorityCreateParams
from .priority_update_params import PriorityUpdateParams as PriorityUpdateParams
from .message_delete_response import MessageDeleteResponse as MessageDeleteResponse
from .priority_delete_response import PriorityDeleteResponse as PriorityDeleteResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["LogoGetParams"]


class LogoGetParams(TypedDict, total=False):
    account_id: Required[str]

    id: str
    """Optional query ID to retrieve a specific logo query"""

    download: str
    """If true, include base64-encoded image data in the response"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["SSL"]


class SSL(BaseModel):
    id: Optional[Literal["ssl"]] = None
    """
    Control options for the SSL feature of the Edge Certificates tab in the
    Cloudflare SSL/TLS app.
    """

    value: Optional[Literal["off", "flexible", "full", "strict", "origin_pull"]] = None
    """The encryption mode that Cloudflare uses to connect to your origin server."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["MetadataIndexCreateParams"]


class MetadataIndexCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier"""

    index_type: Required[Annotated[Literal["string", "number", "boolean"], PropertyInfo(alias="indexType")]]
    """Specifies the type of metadata property to index."""

    property_name: Required[Annotated[str, PropertyInfo(alias="propertyName")]]
    """Specifies the metadata property to index."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._types import FileTypes
from ..._utils import PropertyInfo

__all__ = ["IndexUpsertParams"]


class IndexUpsertParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier"""

    body: Required[FileTypes]
    """ndjson file containing vectors to upsert."""

    unparsable_behavior: Annotated[Literal["error", "discard"], PropertyInfo(alias="unparsable-behavior")]
    """Behavior for ndjson parse failures."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ....._models import BaseModel

__all__ = ["DomainRule", "EmailDomain"]


class EmailDomain(BaseModel):
    domain: str
    """The email domain to match."""


class DomainRule(BaseModel):
    """Match an entire email domain."""

    email_domain: EmailDomain



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from typing_extensions import Literal, Required, TypeAlias, TypedDict

__all__ = [
    "CredentialUpdateParams",
    "Key",
    "KeyAPIShieldCredentialsJWTKeyRSA",
    "KeyAPIShieldCredentialsJWTKeyEcEs256",
    "KeyAPIShieldCredentialsJWTKeyEcEs384",
]


class CredentialUpdateParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    keys: Required[Iterable[Key]]


class KeyAPIShieldCredentialsJWTKeyRSA(TypedDict, total=False):
    """JSON representation of an RSA key."""

    alg: Required[Literal["RS256", "RS384", "RS512", "PS256", "PS384", "PS512"]]
    """Algorithm"""

    e: Required[str]
    """RSA exponent"""

    kid: Required[str]
    """Key ID"""

    kty: Required[Literal["RSA"]]
    """Key Type"""

    n: Required[str]
    """RSA modulus"""


class KeyAPIShieldCredentialsJWTKeyEcEs256(TypedDict, total=False):
    """JSON representation of an ES256 key"""

    alg: Required[Literal["ES256"]]
    """Algorithm"""

    crv: Required[Literal["P-256"]]
    """Curve"""

    kid: Required[str]
    """Key ID"""

    kty: Required[Literal["EC"]]
    """Key Type"""

    x: Required[str]
    """X EC coordinate"""

    y: Required[str]
    """Y EC coordinate"""


class KeyAPIShieldCredentialsJWTKeyEcEs384(TypedDict, total=False):
    """JSON representation of an ES384 key"""

    alg: Required[Literal["ES384"]]
    """Algorithm"""

    crv: Required[Literal["P-384"]]
    """Curve"""

    kid: Required[str]
    """Key ID"""

    kty: Required[Literal["EC"]]
    """Key Type"""

    x: Required[str]
    """X EC coordinate"""

    y: Required[str]
    """Y EC coordinate"""


Key: TypeAlias = Union[
    KeyAPIShieldCredentialsJWTKeyRSA, KeyAPIShieldCredentialsJWTKeyEcEs256, KeyAPIShieldCredentialsJWTKeyEcEs384
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["HistoryUpdateParams"]


class HistoryUpdateParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    body: Required[int]
    """ID of the Zaraz configuration to restore."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .instance_get_params import InstanceGetParams as InstanceGetParams
from .version_list_params import VersionListParams as VersionListParams
from .instance_bulk_params import InstanceBulkParams as InstanceBulkParams
from .instance_list_params import InstanceListParams as InstanceListParams
from .version_get_response import VersionGetResponse as VersionGetResponse
from .workflow_list_params import WorkflowListParams as WorkflowListParams
from .instance_get_response import InstanceGetResponse as InstanceGetResponse
from .version_list_response import VersionListResponse as VersionListResponse
from .workflow_get_response import WorkflowGetResponse as WorkflowGetResponse
from .instance_bulk_response import InstanceBulkResponse as InstanceBulkResponse
from .instance_create_params import InstanceCreateParams as InstanceCreateParams
from .instance_list_response import InstanceListResponse as InstanceListResponse
from .workflow_list_response import WorkflowListResponse as WorkflowListResponse
from .workflow_update_params import WorkflowUpdateParams as WorkflowUpdateParams
from .instance_create_response import InstanceCreateResponse as InstanceCreateResponse
from .workflow_delete_response import WorkflowDeleteResponse as WorkflowDeleteResponse
from .workflow_update_response import WorkflowUpdateResponse as WorkflowUpdateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel

__all__ = ["ScreenshotCreateResponse", "Error"]


class Error(BaseModel):
    code: float
    """Error code."""

    message: str
    """Error message."""


class ScreenshotCreateResponse(BaseModel):
    success: bool
    """Response status."""

    errors: Optional[List[Error]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .web3 import (
    Web3Resource,
    AsyncWeb3Resource,
    Web3ResourceWithRawResponse,
    AsyncWeb3ResourceWithRawResponse,
    Web3ResourceWithStreamingResponse,
    AsyncWeb3ResourceWithStreamingResponse,
)
from .hostnames import (
    HostnamesResource,
    AsyncHostnamesResource,
    HostnamesResourceWithRawResponse,
    AsyncHostnamesResourceWithRawResponse,
    HostnamesResourceWithStreamingResponse,
    AsyncHostnamesResourceWithStreamingResponse,
)

__all__ = [
    "HostnamesResource",
    "AsyncHostnamesResource",
    "HostnamesResourceWithRawResponse",
    "AsyncHostnamesResourceWithRawResponse",
    "HostnamesResourceWithStreamingResponse",
    "AsyncHostnamesResourceWithStreamingResponse",
    "Web3Resource",
    "AsyncWeb3Resource",
    "Web3ResourceWithRawResponse",
    "AsyncWeb3ResourceWithRawResponse",
    "Web3ResourceWithStreamingResponse",
    "AsyncWeb3ResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["RecordImportResponse"]


class RecordImportResponse(BaseModel):
    recs_added: Optional[float] = None
    """Number of DNS records added."""

    total_records_parsed: Optional[float] = None
    """Total number of DNS records parsed."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

__all__ = ["CustomTopicUpdateParams", "Topic"]


class CustomTopicUpdateParams(TypedDict, total=False):
    zone_id: Required[str]
    """Defines the zone."""

    topics: Iterable[Topic]
    """Custom topic categories for AI Security for Apps content detection."""


class Topic(TypedDict, total=False):
    label: Required[str]
    """Unique label identifier.

    Must contain only lowercase letters (a–z), digits (0–9), and hyphens.
    """

    topic: Required[str]
    """Description of the topic category.

    Must contain only printable ASCII characters.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ...._types import Body, Query, Headers, NotGiven, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....pagination import SyncSinglePage, AsyncSinglePage
from ...._base_client import AsyncPaginator, make_request_options
from ....types.dns.zone_transfers import tsig_create_params, tsig_update_params
from ....types.dns.zone_transfers.tsig import TSIG
from ....types.dns.zone_transfers.tsig_delete_response import TSIGDeleteResponse

__all__ = ["TSIGsResource", "AsyncTSIGsResource"]


class TSIGsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TSIGsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return TSIGsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TSIGsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return TSIGsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        algo: str,
        name: str,
        secret: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TSIG]:
        """
        Create TSIG.

        Args:
          algo: TSIG algorithm.

          name: TSIG key name.

          secret: TSIG secret.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/secondary_dns/tsigs", account_id=account_id),
            body=maybe_transform(
                {
                    "algo": algo,
                    "name": name,
                    "secret": secret,
                },
                tsig_create_params.TSIGCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TSIG]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[TSIG]], ResultWrapper[TSIG]),
        )

    def update(
        self,
        tsig_id: str,
        *,
        account_id: str,
        algo: str,
        name: str,
        secret: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TSIG]:
        """
        Modify TSIG.

        Args:
          algo: TSIG algorithm.

          name: TSIG key name.

          secret: TSIG secret.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not tsig_id:
            raise ValueError(f"Expected a non-empty value for `tsig_id` but received {tsig_id!r}")
        return self._put(
            path_template(
                "/accounts/{account_id}/secondary_dns/tsigs/{tsig_id}", account_id=account_id, tsig_id=tsig_id
            ),
            body=maybe_transform(
                {
                    "algo": algo,
                    "name": name,
                    "secret": secret,
                },
                tsig_update_params.TSIGUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TSIG]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[TSIG]], ResultWrapper[TSIG]),
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[TSIG]:
        """
        List TSIGs.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/secondary_dns/tsigs", account_id=account_id),
            page=SyncSinglePage[TSIG],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=TSIG,
        )

    def delete(
        self,
        tsig_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TSIGDeleteResponse]:
        """
        Delete TSIG.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not tsig_id:
            raise ValueError(f"Expected a non-empty value for `tsig_id` but received {tsig_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/secondary_dns/tsigs/{tsig_id}", account_id=account_id, tsig_id=tsig_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TSIGDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[TSIGDeleteResponse]], ResultWrapper[TSIGDeleteResponse]),
        )

    def get(
        self,
        tsig_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TSIG]:
        """
        Get TSIG.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not tsig_id:
            raise ValueError(f"Expected a non-empty value for `tsig_id` but received {tsig_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/secondary_dns/tsigs/{tsig_id}", account_id=account_id, tsig_id=tsig_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TSIG]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[TSIG]], ResultWrapper[TSIG]),
        )


class AsyncTSIGsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTSIGsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTSIGsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTSIGsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncTSIGsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        algo: str,
        name: str,
        secret: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TSIG]:
        """
        Create TSIG.

        Args:
          algo: TSIG algorithm.

          name: TSIG key name.

          secret: TSIG secret.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/secondary_dns/tsigs", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "algo": algo,
                    "name": name,
                    "secret": secret,
                },
                tsig_create_params.TSIGCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TSIG]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[TSIG]], ResultWrapper[TSIG]),
        )

    async def update(
        self,
        tsig_id: str,
        *,
        account_id: str,
        algo: str,
        name: str,
        secret: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TSIG]:
        """
        Modify TSIG.

        Args:
          algo: TSIG algorithm.

          name: TSIG key name.

          secret: TSIG secret.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not tsig_id:
            raise ValueError(f"Expected a non-empty value for `tsig_id` but received {tsig_id!r}")
        return await self._put(
            path_template(
                "/accounts/{account_id}/secondary_dns/tsigs/{tsig_id}", account_id=account_id, tsig_id=tsig_id
            ),
            body=await async_maybe_transform(
                {
                    "algo": algo,
                    "name": name,
                    "secret": secret,
                },
                tsig_update_params.TSIGUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TSIG]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[TSIG]], ResultWrapper[TSIG]),
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[TSIG, AsyncSinglePage[TSIG]]:
        """
        List TSIGs.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/secondary_dns/tsigs", account_id=account_id),
            page=AsyncSinglePage[TSIG],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=TSIG,
        )

    async def delete(
        self,
        tsig_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TSIGDeleteResponse]:
        """
        Delete TSIG.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not tsig_id:
            raise ValueError(f"Expected a non-empty value for `tsig_id` but received {tsig_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/secondary_dns/tsigs/{tsig_id}", account_id=account_id, tsig_id=tsig_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TSIGDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[TSIGDeleteResponse]], ResultWrapper[TSIGDeleteResponse]),
        )

    async def get(
        self,
        tsig_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TSIG]:
        """
        Get TSIG.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not tsig_id:
            raise ValueError(f"Expected a non-empty value for `tsig_id` but received {tsig_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/secondary_dns/tsigs/{tsig_id}", account_id=account_id, tsig_id=tsig_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TSIG]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[TSIG]], ResultWrapper[TSIG]),
        )


class TSIGsResourceWithRawResponse:
    def __init__(self, tsigs: TSIGsResource) -> None:
        self._tsigs = tsigs

        self.create = to_raw_response_wrapper(
            tsigs.create,
        )
        self.update = to_raw_response_wrapper(
            tsigs.update,
        )
        self.list = to_raw_response_wrapper(
            tsigs.list,
        )
        self.delete = to_raw_response_wrapper(
            tsigs.delete,
        )
        self.get = to_raw_response_wrapper(
            tsigs.get,
        )


class AsyncTSIGsResourceWithRawResponse:
    def __init__(self, tsigs: AsyncTSIGsResource) -> None:
        self._tsigs = tsigs

        self.create = async_to_raw_response_wrapper(
            tsigs.create,
        )
        self.update = async_to_raw_response_wrapper(
            tsigs.update,
        )
        self.list = async_to_raw_response_wrapper(
            tsigs.list,
        )
        self.delete = async_to_raw_response_wrapper(
            tsigs.delete,
        )
        self.get = async_to_raw_response_wrapper(
            tsigs.get,
        )


class TSIGsResourceWithStreamingResponse:
    def __init__(self, tsigs: TSIGsResource) -> None:
        self._tsigs = tsigs

        self.create = to_streamed_response_wrapper(
            tsigs.create,
        )
        self.update = to_streamed_response_wrapper(
            tsigs.update,
        )
        self.list = to_streamed_response_wrapper(
            tsigs.list,
        )
        self.delete = to_streamed_response_wrapper(
            tsigs.delete,
        )
        self.get = to_streamed_response_wrapper(
            tsigs.get,
        )


class AsyncTSIGsResourceWithStreamingResponse:
    def __init__(self, tsigs: AsyncTSIGsResource) -> None:
        self._tsigs = tsigs

        self.create = async_to_streamed_response_wrapper(
            tsigs.create,
        )
        self.update = async_to_streamed_response_wrapper(
            tsigs.update,
        )
        self.list = async_to_streamed_response_wrapper(
            tsigs.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            tsigs.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            tsigs.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Union, cast
from datetime import datetime
from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ...._base_client import make_request_options
from ....types.radar.annotations import outage_get_params, outage_locations_params
from ....types.radar.annotations.outage_get_response import OutageGetResponse
from ....types.radar.annotations.outage_locations_response import OutageLocationsResponse

__all__ = ["OutagesResource", "AsyncOutagesResource"]


class OutagesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> OutagesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return OutagesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> OutagesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return OutagesResourceWithStreamingResponse(self)

    def get(
        self,
        *,
        asn: int | Omit = omit,
        date_end: Union[str, datetime] | Omit = omit,
        date_range: str | Omit = omit,
        date_start: Union[str, datetime] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        limit: int | Omit = omit,
        location: str | Omit = omit,
        offset: int | Omit = omit,
        origin: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OutageGetResponse:
        """
        Retrieves the latest Internet outages and anomalies.

        Args:
          asn: Filters results by Autonomous System. Specify a single Autonomous System Number
              (ASN) as integer.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range.

          date_start: Start of the date range (inclusive).

          format: Format in which results will be returned.

          limit: Limits the number of objects returned in the response.

          location: Filters results by location. Specify an alpha-2 location code.

          offset: Skips the specified number of objects before fetching the results.

          origin: Filters results by origin.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/radar/annotations/outages",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "asn": asn,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "limit": limit,
                        "location": location,
                        "offset": offset,
                        "origin": origin,
                    },
                    outage_get_params.OutageGetParams,
                ),
                post_parser=ResultWrapper[OutageGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[OutageGetResponse], ResultWrapper[OutageGetResponse]),
        )

    def locations(
        self,
        *,
        date_end: Union[str, datetime] | Omit = omit,
        date_range: str | Omit = omit,
        date_start: Union[str, datetime] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        limit: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OutageLocationsResponse:
        """
        Retrieves the number of outages by location.

        Args:
          date_end: End of the date range (inclusive).

          date_range: Filters results by date range.

          date_start: Start of the date range (inclusive).

          format: Format in which results will be returned.

          limit: Limits the number of objects returned in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/radar/annotations/outages/locations",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "limit": limit,
                    },
                    outage_locations_params.OutageLocationsParams,
                ),
                post_parser=ResultWrapper[OutageLocationsResponse]._unwrapper,
            ),
            cast_to=cast(Type[OutageLocationsResponse], ResultWrapper[OutageLocationsResponse]),
        )


class AsyncOutagesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncOutagesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncOutagesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncOutagesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncOutagesResourceWithStreamingResponse(self)

    async def get(
        self,
        *,
        asn: int | Omit = omit,
        date_end: Union[str, datetime] | Omit = omit,
        date_range: str | Omit = omit,
        date_start: Union[str, datetime] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        limit: int | Omit = omit,
        location: str | Omit = omit,
        offset: int | Omit = omit,
        origin: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OutageGetResponse:
        """
        Retrieves the latest Internet outages and anomalies.

        Args:
          asn: Filters results by Autonomous System. Specify a single Autonomous System Number
              (ASN) as integer.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range.

          date_start: Start of the date range (inclusive).

          format: Format in which results will be returned.

          limit: Limits the number of objects returned in the response.

          location: Filters results by location. Specify an alpha-2 location code.

          offset: Skips the specified number of objects before fetching the results.

          origin: Filters results by origin.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/radar/annotations/outages",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "asn": asn,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "limit": limit,
                        "location": location,
                        "offset": offset,
                        "origin": origin,
                    },
                    outage_get_params.OutageGetParams,
                ),
                post_parser=ResultWrapper[OutageGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[OutageGetResponse], ResultWrapper[OutageGetResponse]),
        )

    async def locations(
        self,
        *,
        date_end: Union[str, datetime] | Omit = omit,
        date_range: str | Omit = omit,
        date_start: Union[str, datetime] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        limit: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OutageLocationsResponse:
        """
        Retrieves the number of outages by location.

        Args:
          date_end: End of the date range (inclusive).

          date_range: Filters results by date range.

          date_start: Start of the date range (inclusive).

          format: Format in which results will be returned.

          limit: Limits the number of objects returned in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/radar/annotations/outages/locations",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "limit": limit,
                    },
                    outage_locations_params.OutageLocationsParams,
                ),
                post_parser=ResultWrapper[OutageLocationsResponse]._unwrapper,
            ),
            cast_to=cast(Type[OutageLocationsResponse], ResultWrapper[OutageLocationsResponse]),
        )


class OutagesResourceWithRawResponse:
    def __init__(self, outages: OutagesResource) -> None:
        self._outages = outages

        self.get = to_raw_response_wrapper(
            outages.get,
        )
        self.locations = to_raw_response_wrapper(
            outages.locations,
        )


class AsyncOutagesResourceWithRawResponse:
    def __init__(self, outages: AsyncOutagesResource) -> None:
        self._outages = outages

        self.get = async_to_raw_response_wrapper(
            outages.get,
        )
        self.locations = async_to_raw_response_wrapper(
            outages.locations,
        )


class OutagesResourceWithStreamingResponse:
    def __init__(self, outages: OutagesResource) -> None:
        self._outages = outages

        self.get = to_streamed_response_wrapper(
            outages.get,
        )
        self.locations = to_streamed_response_wrapper(
            outages.locations,
        )


class AsyncOutagesResourceWithStreamingResponse:
    def __init__(self, outages: AsyncOutagesResource) -> None:
        self._outages = outages

        self.get = async_to_streamed_response_wrapper(
            outages.get,
        )
        self.locations = async_to_streamed_response_wrapper(
            outages.locations,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ..._models import BaseModel

__all__ = ["StoreCreateResponse"]


class StoreCreateResponse(BaseModel):
    id: str
    """Store Identifier"""

    created: datetime
    """Whenthe secret was created."""

    modified: datetime
    """When the secret was modified."""

    name: str
    """The name of the store"""

    account_id: Optional[str] = None
    """Account Identifier"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.cloudforce_one.threat_events import raw_edit_params
from ....types.cloudforce_one.threat_events.raw_get_response import RawGetResponse
from ....types.cloudforce_one.threat_events.raw_edit_response import RawEditResponse

__all__ = ["RawResource", "AsyncRawResource"]


class RawResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> RawResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return RawResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RawResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return RawResourceWithStreamingResponse(self)

    def edit(
        self,
        raw_id: str,
        *,
        account_id: str,
        event_id: str,
        data: object | Omit = omit,
        source: str | Omit = omit,
        tlp: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RawEditResponse:
        """
        Updates a raw event

        Args:
          account_id: Account ID.

          event_id: Event UUID.

          raw_id: Raw Event UUID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not event_id:
            raise ValueError(f"Expected a non-empty value for `event_id` but received {event_id!r}")
        if not raw_id:
            raise ValueError(f"Expected a non-empty value for `raw_id` but received {raw_id!r}")
        return self._patch(
            path_template(
                "/accounts/{account_id}/cloudforce-one/events/{event_id}/raw/{raw_id}",
                account_id=account_id,
                event_id=event_id,
                raw_id=raw_id,
            ),
            body=maybe_transform(
                {
                    "data": data,
                    "source": source,
                    "tlp": tlp,
                },
                raw_edit_params.RawEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RawEditResponse,
        )

    def get(
        self,
        raw_id: str,
        *,
        account_id: str,
        event_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RawGetResponse:
        """
        Reads data for a raw event

        Args:
          account_id: Account ID.

          event_id: Event UUID.

          raw_id: Raw Event UUID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not event_id:
            raise ValueError(f"Expected a non-empty value for `event_id` but received {event_id!r}")
        if not raw_id:
            raise ValueError(f"Expected a non-empty value for `raw_id` but received {raw_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/cloudforce-one/events/{event_id}/raw/{raw_id}",
                account_id=account_id,
                event_id=event_id,
                raw_id=raw_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RawGetResponse,
        )


class AsyncRawResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncRawResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRawResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRawResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncRawResourceWithStreamingResponse(self)

    async def edit(
        self,
        raw_id: str,
        *,
        account_id: str,
        event_id: str,
        data: object | Omit = omit,
        source: str | Omit = omit,
        tlp: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RawEditResponse:
        """
        Updates a raw event

        Args:
          account_id: Account ID.

          event_id: Event UUID.

          raw_id: Raw Event UUID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not event_id:
            raise ValueError(f"Expected a non-empty value for `event_id` but received {event_id!r}")
        if not raw_id:
            raise ValueError(f"Expected a non-empty value for `raw_id` but received {raw_id!r}")
        return await self._patch(
            path_template(
                "/accounts/{account_id}/cloudforce-one/events/{event_id}/raw/{raw_id}",
                account_id=account_id,
                event_id=event_id,
                raw_id=raw_id,
            ),
            body=await async_maybe_transform(
                {
                    "data": data,
                    "source": source,
                    "tlp": tlp,
                },
                raw_edit_params.RawEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RawEditResponse,
        )

    async def get(
        self,
        raw_id: str,
        *,
        account_id: str,
        event_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RawGetResponse:
        """
        Reads data for a raw event

        Args:
          account_id: Account ID.

          event_id: Event UUID.

          raw_id: Raw Event UUID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not event_id:
            raise ValueError(f"Expected a non-empty value for `event_id` but received {event_id!r}")
        if not raw_id:
            raise ValueError(f"Expected a non-empty value for `raw_id` but received {raw_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/cloudforce-one/events/{event_id}/raw/{raw_id}",
                account_id=account_id,
                event_id=event_id,
                raw_id=raw_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RawGetResponse,
        )


class RawResourceWithRawResponse:
    def __init__(self, raw: RawResource) -> None:
        self._raw = raw

        self.edit = to_raw_response_wrapper(
            raw.edit,
        )
        self.get = to_raw_response_wrapper(
            raw.get,
        )


class AsyncRawResourceWithRawResponse:
    def __init__(self, raw: AsyncRawResource) -> None:
        self._raw = raw

        self.edit = async_to_raw_response_wrapper(
            raw.edit,
        )
        self.get = async_to_raw_response_wrapper(
            raw.get,
        )


class RawResourceWithStreamingResponse:
    def __init__(self, raw: RawResource) -> None:
        self._raw = raw

        self.edit = to_streamed_response_wrapper(
            raw.edit,
        )
        self.get = to_streamed_response_wrapper(
            raw.get,
        )


class AsyncRawResourceWithStreamingResponse:
    def __init__(self, raw: AsyncRawResource) -> None:
        self._raw = raw

        self.edit = async_to_streamed_response_wrapper(
            raw.edit,
        )
        self.get = async_to_streamed_response_wrapper(
            raw.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .browser import (
    BrowserResource,
    AsyncBrowserResource,
    BrowserResourceWithRawResponse,
    AsyncBrowserResourceWithRawResponse,
    BrowserResourceWithStreamingResponse,
    AsyncBrowserResourceWithStreamingResponse,
)
from .session import (
    SessionResource,
    AsyncSessionResource,
    SessionResourceWithRawResponse,
    AsyncSessionResourceWithRawResponse,
    SessionResourceWithStreamingResponse,
    AsyncSessionResourceWithStreamingResponse,
)
from .devtools import (
    DevtoolsResource,
    AsyncDevtoolsResource,
    DevtoolsResourceWithRawResponse,
    AsyncDevtoolsResourceWithRawResponse,
    DevtoolsResourceWithStreamingResponse,
    AsyncDevtoolsResourceWithStreamingResponse,
)

__all__ = [
    "SessionResource",
    "AsyncSessionResource",
    "SessionResourceWithRawResponse",
    "AsyncSessionResourceWithRawResponse",
    "SessionResourceWithStreamingResponse",
    "AsyncSessionResourceWithStreamingResponse",
    "BrowserResource",
    "AsyncBrowserResource",
    "BrowserResourceWithRawResponse",
    "AsyncBrowserResourceWithRawResponse",
    "BrowserResourceWithStreamingResponse",
    "AsyncBrowserResourceWithStreamingResponse",
    "DevtoolsResource",
    "AsyncDevtoolsResource",
    "DevtoolsResourceWithRawResponse",
    "AsyncDevtoolsResourceWithRawResponse",
    "DevtoolsResourceWithStreamingResponse",
    "AsyncDevtoolsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from .host import Host
from ..._types import SequenceNotStr
from ..._utils import PropertyInfo

__all__ = ["HeaderParam"]


class HeaderParam(TypedDict, total=False):
    """The request header is used to pass additional information with an HTTP request.

    Currently supported header is 'Host'.
    """

    host: Annotated[SequenceNotStr[Host], PropertyInfo(alias="Host")]
    """The 'Host' header allows to override the hostname set in the HTTP request.

    Current support is 1 'Host' header override per origin.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from .events.events import (
    EventsResource,
    AsyncEventsResource,
    EventsResourceWithRawResponse,
    AsyncEventsResourceWithRawResponse,
    EventsResourceWithStreamingResponse,
    AsyncEventsResourceWithStreamingResponse,
)
from .aggregates.aggregates import (
    AggregatesResource,
    AsyncAggregatesResource,
    AggregatesResourceWithRawResponse,
    AsyncAggregatesResourceWithRawResponse,
    AggregatesResourceWithStreamingResponse,
    AsyncAggregatesResourceWithStreamingResponse,
)

__all__ = ["AnalyticsResource", "AsyncAnalyticsResource"]


class AnalyticsResource(SyncAPIResource):
    @cached_property
    def aggregates(self) -> AggregatesResource:
        return AggregatesResource(self._client)

    @cached_property
    def events(self) -> EventsResource:
        return EventsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AnalyticsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AnalyticsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AnalyticsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AnalyticsResourceWithStreamingResponse(self)


class AsyncAnalyticsResource(AsyncAPIResource):
    @cached_property
    def aggregates(self) -> AsyncAggregatesResource:
        return AsyncAggregatesResource(self._client)

    @cached_property
    def events(self) -> AsyncEventsResource:
        return AsyncEventsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncAnalyticsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAnalyticsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAnalyticsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncAnalyticsResourceWithStreamingResponse(self)


class AnalyticsResourceWithRawResponse:
    def __init__(self, analytics: AnalyticsResource) -> None:
        self._analytics = analytics

    @cached_property
    def aggregates(self) -> AggregatesResourceWithRawResponse:
        return AggregatesResourceWithRawResponse(self._analytics.aggregates)

    @cached_property
    def events(self) -> EventsResourceWithRawResponse:
        return EventsResourceWithRawResponse(self._analytics.events)


class AsyncAnalyticsResourceWithRawResponse:
    def __init__(self, analytics: AsyncAnalyticsResource) -> None:
        self._analytics = analytics

    @cached_property
    def aggregates(self) -> AsyncAggregatesResourceWithRawResponse:
        return AsyncAggregatesResourceWithRawResponse(self._analytics.aggregates)

    @cached_property
    def events(self) -> AsyncEventsResourceWithRawResponse:
        return AsyncEventsResourceWithRawResponse(self._analytics.events)


class AnalyticsResourceWithStreamingResponse:
    def __init__(self, analytics: AnalyticsResource) -> None:
        self._analytics = analytics

    @cached_property
    def aggregates(self) -> AggregatesResourceWithStreamingResponse:
        return AggregatesResourceWithStreamingResponse(self._analytics.aggregates)

    @cached_property
    def events(self) -> EventsResourceWithStreamingResponse:
        return EventsResourceWithStreamingResponse(self._analytics.events)


class AsyncAnalyticsResourceWithStreamingResponse:
    def __init__(self, analytics: AsyncAnalyticsResource) -> None:
        self._analytics = analytics

    @cached_property
    def aggregates(self) -> AsyncAggregatesResourceWithStreamingResponse:
        return AsyncAggregatesResourceWithStreamingResponse(self._analytics.aggregates)

    @cached_property
    def events(self) -> AsyncEventsResourceWithStreamingResponse:
        return AsyncEventsResourceWithStreamingResponse(self._analytics.events)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .fallthrough import (
    FallthroughResource,
    AsyncFallthroughResource,
    FallthroughResourceWithRawResponse,
    AsyncFallthroughResourceWithRawResponse,
    FallthroughResourceWithStreamingResponse,
    AsyncFallthroughResourceWithStreamingResponse,
)
from .expression_template import (
    ExpressionTemplateResource,
    AsyncExpressionTemplateResource,
    ExpressionTemplateResourceWithRawResponse,
    AsyncExpressionTemplateResourceWithRawResponse,
    ExpressionTemplateResourceWithStreamingResponse,
    AsyncExpressionTemplateResourceWithStreamingResponse,
)

__all__ = [
    "FallthroughResource",
    "AsyncFallthroughResource",
    "FallthroughResourceWithRawResponse",
    "AsyncFallthroughResourceWithRawResponse",
    "FallthroughResourceWithStreamingResponse",
    "AsyncFallthroughResourceWithStreamingResponse",
    "ExpressionTemplateResource",
    "AsyncExpressionTemplateResource",
    "ExpressionTemplateResourceWithRawResponse",
    "AsyncExpressionTemplateResourceWithRawResponse",
    "ExpressionTemplateResourceWithStreamingResponse",
    "AsyncExpressionTemplateResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ...._models import BaseModel

__all__ = [
    "LoggingSetting",
    "SettingsByRuleType",
    "SettingsByRuleTypeDNS",
    "SettingsByRuleTypeHTTP",
    "SettingsByRuleTypeL4",
]


class SettingsByRuleTypeDNS(BaseModel):
    """Configure logging settings for DNS firewall."""

    log_all: Optional[bool] = None
    """Specify whether to log all requests to this service."""

    log_blocks: Optional[bool] = None
    """Specify whether to log only blocking requests to this service."""


class SettingsByRuleTypeHTTP(BaseModel):
    """Configure logging settings for HTTP/HTTPS firewall."""

    log_all: Optional[bool] = None
    """Specify whether to log all requests to this service."""

    log_blocks: Optional[bool] = None
    """Specify whether to log only blocking requests to this service."""


class SettingsByRuleTypeL4(BaseModel):
    """Configure logging settings for Network firewall."""

    log_all: Optional[bool] = None
    """Specify whether to log all requests to this service."""

    log_blocks: Optional[bool] = None
    """Specify whether to log only blocking requests to this service."""


class SettingsByRuleType(BaseModel):
    """Configure logging settings for each rule type."""

    dns: Optional[SettingsByRuleTypeDNS] = None
    """Configure logging settings for DNS firewall."""

    http: Optional[SettingsByRuleTypeHTTP] = None
    """Configure logging settings for HTTP/HTTPS firewall."""

    l4: Optional[SettingsByRuleTypeL4] = None
    """Configure logging settings for Network firewall."""


class LoggingSetting(BaseModel):
    redact_pii: Optional[bool] = None
    """
    Indicate whether to redact personally identifiable information from activity
    logging (PII fields include source IP, user email, user ID, device ID, URL,
    referrer, and user agent).
    """

    settings_by_rule_type: Optional[SettingsByRuleType] = None
    """Configure logging settings for each rule type."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncCursorPaginationAfter, AsyncCursorPaginationAfter
from ..._base_client import AsyncPaginator, make_request_options
from ...types.resource_tagging import value_list_params
from ...types.resource_tagging.value_list_response import ValueListResponse

__all__ = ["ValuesResource", "AsyncValuesResource"]


class ValuesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ValuesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return ValuesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ValuesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return ValuesResourceWithStreamingResponse(self)

    def list(
        self,
        tag_key: str,
        *,
        account_id: str,
        cursor: str | Omit = omit,
        type: Literal[
            "access_application",
            "access_application_policy",
            "access_group",
            "account",
            "ai_gateway",
            "alerting_policy",
            "alerting_webhook",
            "api_gateway_operation",
            "cloudflared_tunnel",
            "custom_certificate",
            "custom_hostname",
            "d1_database",
            "dns_record",
            "durable_object_namespace",
            "gateway_list",
            "gateway_rule",
            "image",
            "kv_namespace",
            "managed_client_certificate",
            "queue",
            "r2_bucket",
            "resource_share",
            "stream_live_input",
            "stream_video",
            "worker",
            "worker_version",
            "zone",
        ]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorPaginationAfter[ValueListResponse]:
        """
        Lists all distinct values for a given tag key, optionally filtered by resource
        type.

        Args:
          account_id: Identifier.

          cursor: Cursor for pagination.

          type: Filter by resource type.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not tag_key:
            raise ValueError(f"Expected a non-empty value for `tag_key` but received {tag_key!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/tags/values/{tag_key}", account_id=account_id, tag_key=tag_key),
            page=SyncCursorPaginationAfter[ValueListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "type": type,
                    },
                    value_list_params.ValueListParams,
                ),
            ),
            model=str,
        )


class AsyncValuesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncValuesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncValuesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncValuesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncValuesResourceWithStreamingResponse(self)

    def list(
        self,
        tag_key: str,
        *,
        account_id: str,
        cursor: str | Omit = omit,
        type: Literal[
            "access_application",
            "access_application_policy",
            "access_group",
            "account",
            "ai_gateway",
            "alerting_policy",
            "alerting_webhook",
            "api_gateway_operation",
            "cloudflared_tunnel",
            "custom_certificate",
            "custom_hostname",
            "d1_database",
            "dns_record",
            "durable_object_namespace",
            "gateway_list",
            "gateway_rule",
            "image",
            "kv_namespace",
            "managed_client_certificate",
            "queue",
            "r2_bucket",
            "resource_share",
            "stream_live_input",
            "stream_video",
            "worker",
            "worker_version",
            "zone",
        ]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[ValueListResponse, AsyncCursorPaginationAfter[ValueListResponse]]:
        """
        Lists all distinct values for a given tag key, optionally filtered by resource
        type.

        Args:
          account_id: Identifier.

          cursor: Cursor for pagination.

          type: Filter by resource type.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not tag_key:
            raise ValueError(f"Expected a non-empty value for `tag_key` but received {tag_key!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/tags/values/{tag_key}", account_id=account_id, tag_key=tag_key),
            page=AsyncCursorPaginationAfter[ValueListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "type": type,
                    },
                    value_list_params.ValueListParams,
                ),
            ),
            model=str,
        )


class ValuesResourceWithRawResponse:
    def __init__(self, values: ValuesResource) -> None:
        self._values = values

        self.list = to_raw_response_wrapper(
            values.list,
        )


class AsyncValuesResourceWithRawResponse:
    def __init__(self, values: AsyncValuesResource) -> None:
        self._values = values

        self.list = async_to_raw_response_wrapper(
            values.list,
        )


class ValuesResourceWithStreamingResponse:
    def __init__(self, values: ValuesResource) -> None:
        self._values = values

        self.list = to_streamed_response_wrapper(
            values.list,
        )


class AsyncValuesResourceWithStreamingResponse:
    def __init__(self, values: AsyncValuesResource) -> None:
        self._values = values

        self.list = async_to_streamed_response_wrapper(
            values.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ...._models import BaseModel

__all__ = ["ScanResult"]


class ScanResult(BaseModel):
    number: Optional[float] = None

    proto: Optional[str] = None

    status: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .email import (
    EmailResource,
    AsyncEmailResource,
    EmailResourceWithRawResponse,
    AsyncEmailResourceWithRawResponse,
    EmailResourceWithStreamingResponse,
    AsyncEmailResourceWithStreamingResponse,
)
from .rules import (
    RulesResource,
    AsyncRulesResource,
    RulesResourceWithRawResponse,
    AsyncRulesResourceWithRawResponse,
    RulesResourceWithStreamingResponse,
    AsyncRulesResourceWithStreamingResponse,
)
from .account_mapping import (
    AccountMappingResource,
    AsyncAccountMappingResource,
    AccountMappingResourceWithRawResponse,
    AsyncAccountMappingResourceWithRawResponse,
    AccountMappingResourceWithStreamingResponse,
    AsyncAccountMappingResourceWithStreamingResponse,
)

__all__ = [
    "AccountMappingResource",
    "AsyncAccountMappingResource",
    "AccountMappingResourceWithRawResponse",
    "AsyncAccountMappingResourceWithRawResponse",
    "AccountMappingResourceWithStreamingResponse",
    "AsyncAccountMappingResourceWithStreamingResponse",
    "RulesResource",
    "AsyncRulesResource",
    "RulesResourceWithRawResponse",
    "AsyncRulesResourceWithRawResponse",
    "RulesResourceWithStreamingResponse",
    "AsyncRulesResourceWithStreamingResponse",
    "EmailResource",
    "AsyncEmailResource",
    "EmailResourceWithRawResponse",
    "AsyncEmailResourceWithRawResponse",
    "EmailResourceWithStreamingResponse",
    "AsyncEmailResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast
from typing_extensions import Literal

import httpx

from .secrets import (
    SecretsResource,
    AsyncSecretsResource,
    SecretsResourceWithRawResponse,
    AsyncSecretsResourceWithRawResponse,
    SecretsResourceWithStreamingResponse,
    AsyncSecretsResourceWithStreamingResponse,
)
from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ...._base_client import AsyncPaginator, make_request_options
from ....types.secrets_store import store_list_params, store_create_params, store_delete_params
from ....types.secrets_store.store_list_response import StoreListResponse
from ....types.secrets_store.store_create_response import StoreCreateResponse

__all__ = ["StoresResource", "AsyncStoresResource"]


class StoresResource(SyncAPIResource):
    @cached_property
    def secrets(self) -> SecretsResource:
        return SecretsResource(self._client)

    @cached_property
    def with_raw_response(self) -> StoresResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return StoresResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> StoresResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return StoresResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[StoreCreateResponse]:
        """
        Creates a store in the account

        Args:
          account_id: Account Identifier

          name: The name of the store

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/secrets_store/stores", account_id=account_id),
            body=maybe_transform({"name": name}, store_create_params.StoreCreateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[StoreCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[StoreCreateResponse]], ResultWrapper[StoreCreateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        direction: Literal["asc", "desc"] | Omit = omit,
        order: Literal["name", "comment", "created", "modified", "status"] | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[StoreListResponse]:
        """
        Lists all the stores in an account

        Args:
          account_id: Account Identifier

          direction: Direction to sort objects

          order: Order secrets by values in the given field

          page: Page number

          per_page: Number of objects to return per page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/secrets_store/stores", account_id=account_id),
            page=SyncV4PagePaginationArray[StoreListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "direction": direction,
                        "order": order,
                        "page": page,
                        "per_page": per_page,
                    },
                    store_list_params.StoreListParams,
                ),
            ),
            model=StoreListResponse,
        )

    def delete(
        self,
        store_id: str,
        *,
        account_id: str,
        force: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """Deletes a single store.

        By default, a store that still contains secrets cannot
        be deleted and returns HTTP 409 (Conflict) with the "store_not_empty" error.
        Pass `force=true` to cascade-delete all secrets in the store. Empty stores are
        always deleted regardless of the force parameter.

        Args:
          account_id: Account Identifier

          store_id: Store Identifier

          force: When true, cascade-deletes all secrets in the store before deleting the store
              itself. Required when deleting a non-empty store. Without this parameter,
              attempting to delete a non-empty store returns 409.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not store_id:
            raise ValueError(f"Expected a non-empty value for `store_id` but received {store_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/secrets_store/stores/{store_id}", account_id=account_id, store_id=store_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"force": force}, store_delete_params.StoreDeleteParams),
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )


class AsyncStoresResource(AsyncAPIResource):
    @cached_property
    def secrets(self) -> AsyncSecretsResource:
        return AsyncSecretsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncStoresResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncStoresResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncStoresResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncStoresResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[StoreCreateResponse]:
        """
        Creates a store in the account

        Args:
          account_id: Account Identifier

          name: The name of the store

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/secrets_store/stores", account_id=account_id),
            body=await async_maybe_transform({"name": name}, store_create_params.StoreCreateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[StoreCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[StoreCreateResponse]], ResultWrapper[StoreCreateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        direction: Literal["asc", "desc"] | Omit = omit,
        order: Literal["name", "comment", "created", "modified", "status"] | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[StoreListResponse, AsyncV4PagePaginationArray[StoreListResponse]]:
        """
        Lists all the stores in an account

        Args:
          account_id: Account Identifier

          direction: Direction to sort objects

          order: Order secrets by values in the given field

          page: Page number

          per_page: Number of objects to return per page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/secrets_store/stores", account_id=account_id),
            page=AsyncV4PagePaginationArray[StoreListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "direction": direction,
                        "order": order,
                        "page": page,
                        "per_page": per_page,
                    },
                    store_list_params.StoreListParams,
                ),
            ),
            model=StoreListResponse,
        )

    async def delete(
        self,
        store_id: str,
        *,
        account_id: str,
        force: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """Deletes a single store.

        By default, a store that still contains secrets cannot
        be deleted and returns HTTP 409 (Conflict) with the "store_not_empty" error.
        Pass `force=true` to cascade-delete all secrets in the store. Empty stores are
        always deleted regardless of the force parameter.

        Args:
          account_id: Account Identifier

          store_id: Store Identifier

          force: When true, cascade-deletes all secrets in the store before deleting the store
              itself. Required when deleting a non-empty store. Without this parameter,
              attempting to delete a non-empty store returns 409.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not store_id:
            raise ValueError(f"Expected a non-empty value for `store_id` but received {store_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/secrets_store/stores/{store_id}", account_id=account_id, store_id=store_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"force": force}, store_delete_params.StoreDeleteParams),
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )


class StoresResourceWithRawResponse:
    def __init__(self, stores: StoresResource) -> None:
        self._stores = stores

        self.create = to_raw_response_wrapper(
            stores.create,
        )
        self.list = to_raw_response_wrapper(
            stores.list,
        )
        self.delete = to_raw_response_wrapper(
            stores.delete,
        )

    @cached_property
    def secrets(self) -> SecretsResourceWithRawResponse:
        return SecretsResourceWithRawResponse(self._stores.secrets)


class AsyncStoresResourceWithRawResponse:
    def __init__(self, stores: AsyncStoresResource) -> None:
        self._stores = stores

        self.create = async_to_raw_response_wrapper(
            stores.create,
        )
        self.list = async_to_raw_response_wrapper(
            stores.list,
        )
        self.delete = async_to_raw_response_wrapper(
            stores.delete,
        )

    @cached_property
    def secrets(self) -> AsyncSecretsResourceWithRawResponse:
        return AsyncSecretsResourceWithRawResponse(self._stores.secrets)


class StoresResourceWithStreamingResponse:
    def __init__(self, stores: StoresResource) -> None:
        self._stores = stores

        self.create = to_streamed_response_wrapper(
            stores.create,
        )
        self.list = to_streamed_response_wrapper(
            stores.list,
        )
        self.delete = to_streamed_response_wrapper(
            stores.delete,
        )

    @cached_property
    def secrets(self) -> SecretsResourceWithStreamingResponse:
        return SecretsResourceWithStreamingResponse(self._stores.secrets)


class AsyncStoresResourceWithStreamingResponse:
    def __init__(self, stores: AsyncStoresResource) -> None:
        self._stores = stores

        self.create = async_to_streamed_response_wrapper(
            stores.create,
        )
        self.list = async_to_streamed_response_wrapper(
            stores.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            stores.delete,
        )

    @cached_property
    def secrets(self) -> AsyncSecretsResourceWithStreamingResponse:
        return AsyncSecretsResourceWithStreamingResponse(self._stores.secrets)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["TCPConfigurationParam"]


class TCPConfigurationParam(TypedDict, total=False):
    """Parameters specific to TCP health check."""

    method: Literal["connection_established"]
    """The TCP connection method to use for the health check."""

    port: int
    """Port number to connect to for the health check. Defaults to 80."""
